
package simula.random;

import java.util.Random;

/**
 * The class Discrete allows to draw integer values distributed according
 * to an arbitrary discrete distribution. The values extracted are always
 * in the range 0..n.
 */
public class Discrete extends Generator {

	private double[] cumul;


	/**
	 * Creates a new generator using the java standard engine and the current
	 * time as initial seed.
	 * @param c    Array that contains a discrete distribution.
	 */
	public Discrete(double[] c) {
		random = new RandomEngineJava();
		setParam(c);
	}
	/**
	 * Creates a new generator using the java standard engine.
	 * @param seed Initial seed to the pseudo-random engine.
	 * @param c    Array that contains a discrete distribution.
	 */
	public Discrete(long seed, double[] c) {
		random = new RandomEngineJava(seed);
		setParam(c);
	}
	/**
	 * Creates a new generator using the given engine.
	 * @param e Pseudo-random engine to use.
	 * @param c Array that contains a discrete distribution.
	 */
	public Discrete(RandomEngine e,double[] c) {
		random = e;
		setParam(c);
	}
	/**
	 * Draws a value from the generator.
	 * @return An instance of Value that encapsulates an integer value.
	 * @see IntegerValue
	 * @see Value
	 */
	public Value draw() {
		double l = random.nextDouble();
		int i = 0;
		
		while (cumul[i] < l) i++;
		return new IntegerValue( i );
	}
	/**
	 * Change the discrete distribution..
	 * @param c Array that contains a discrete distribution.
	 */
	public void setParam(double[] c) {
		cumul = new double[c.length];
		for(int i=0;i<c.length;i++)
			cumul[i]=c[i];
	}
	/**
	 * Returns the type of the extracted value.
	 */
	public int type() {
		return INTEGER_TYPE;
	}
}