
package simula.random;

import java.util.Random;

/**
 * The class Histd allows to draw integer values distributed in the range 0..n
 * according to a known histogram of the absolute frequencies. Useful to
 * extract numbers which we know the empiric distribution of.
 */
public class Histd extends Generator
{
/**
** Contains the normalized cumulative value.
*/
  private double[] b;

  private Discrete d;
  

/**            
 * Creates a new Histd generator using the java standard engine.
 * @param f    Array that contains the absolute frequencies.
 */  
  public Histd(double[] f)
  {
	random = new RandomEngineJava();
	setParam(f);    
  }  
/**
 * Creates a new Histd generator using the java standard engine and the given seed.
 * @param seed Initial seed to the pseudo-random engine.
 * @param f    Array that contains the absolute frequencies.
 */
  public Histd(long seed,double[] f)
  {
	random = new RandomEngineJava(seed);
	setParam(f);
  }  
/**            
 * Creates a new Histd generator using the given engine.
 * @param e    Pseudo-random engine to use.
 * @param f    Array that contains the absolute frequencies.
 */  
  public Histd(RandomEngine e,double[] f)
  {
	random = e;
		setParam(f);
  }  
/**
 * Draws a value from the generator.
 * @return An instance of Value that encapsulates a real value.
 * @see IntegerValue
 * @see Value
 */
  public Value draw()
  {
	return d.draw();
  }  
/**
 * Change the histogram of the absolute frequencies.
 * @param f    Array that contains the absolute frequencies.
 */  
  public void setParam(double[] a)
  {
	double sum = 0.0;
	
	for(int i=0;i<a.length;i++)
	  sum += a[i];
	  
	for(int i=0;i<a.length;i++)
	{
	  for(int j=0;j<=i;j++)
		b[i] += b[j];
		
	  b[i] /= sum;  
	} 
	
	d = new Discrete(random,b);
  }  
/**
 * Returns the type of the extracted value.
 */
  public int type()
  {
  	return d.type();
  }  
}