
package simula.random;

import java.util.Random;

/**
 * The class Negexp allows to draw real values according to an exponential
 * distribution.
 */
public class Negexp extends Generator {

/**
** Parameter of the exponential.
*/  
  private double lambda;


/**
 * Creates a new exponential generator using the standard engine and the current time as seed.
 * @param l    Parameter of the exponential random variable.
 */
  public Negexp(double l)
  {
	random = new RandomEngineJava();
	lambda = l;
  }  
/**
 * Creates a new exponential generator using the standard engine and the given seed.
 * @param seed Initial seed for the pseudo-random engine.
 * @param l    Parameter of the exponential random variable.
 */
  public Negexp(long seed,double l)
  {
	random = new RandomEngineJava(seed);
	lambda = l;
  }  
/**
 * Creates a new exponential generator using the given engine.
 * @param e    Pseudo-random bit generator.
 * @param l    Parameter of the exponential random variable.
 */
  public Negexp(RandomEngine e,double l)
  {
	random = e;
	lambda = l;
  }  
/**
 * Returns the value of the density in the given point.
 * @param x     Point to value the density of.
 * @return the value of the density in the given point.
 */
  public double density (double x)
  {
	double res=-1;

	if (x>=0) 
	  res=lambda*Math.exp(-lambda*x);
	return res;
	

  }  
/**
 * Draws a value from the generator.
 * @return An instance of Value that encapsulates a real value.
 * @see RealValue
 * @see Value
 */
  public Value draw()
  {
	return new RealValue( Math.log(1-random.nextDouble())/(-lambda)  );
  }  
/**
 * Change the parameter of the exponential.
 * @param l    Parameter of the exponential random variable.
 */
  public void setParam(double l)
  {
	lambda = l;
  }  
/**
 * Returns the type of the extracted value.
 */
  public int type()
  {
  	return REAL_TYPE;
  }  
}