
package simula.random;

import java.util.Random;

/**
 * The class Normal allows to generate real values with a normal distribution.
 * @author Dalcorso Luca, Fin Alessandro, Riva Nicola
 */
public class Normal extends Generator
{
/**
** Average.
*/
  private double mi ;

/**
** Standard deviation.
*/
  private double sigma ;


/**
 * Creates a new Normal generator using the standard engine and the current time as seed.
** @param m    Average.
** @param s    Square root of the standard deviation.
*/                                                       
  public Normal(double m,double s)
  {
	random = new RandomEngineJava();
	mi = m;
	sigma = s;
  }   
/**
 * Creates a new Normal generator using the standard engine with the given seed.
** @param seed Initial seed for the pseudo-random engine.
** @param m    Average.
** @param s    Square root of the standard deviation.
*/
  public Normal(long seed,double m,double s)
  {
	random = new RandomEngineJava(seed);
	mi = m;
	sigma = s;
  }  
/**
 * Creates a new Normal generator using the given engine.
** @param e    Pseudo-random bit generator.
** @param m    Average.
** @param s    Square root of the standard deviation.
** @see RandomEngine
*/                                                       
  public Normal(RandomEngine e,double m,double s)
  {
	random = e;
	mi = m;
	sigma = s;
  }   
/**
 * Returns the value of the density in the given point.
 * @param x     Point to value the density of.
 * @return the value of the density in the given point.
 */	
  public double density (double x) 
  {
	 double z = (x-mi)/sigma;
	 return 1.0 / (Math.sqrt(2*Math.PI)*sigma) * Math.exp(-(z)*(z));


  }  
/**
 * Draws a value from the generator.
 * @return An instance of Value that encapsulates a real value.
 * @see RealValue
 * @see Value
*/
  public Value draw()
  {
	return new RealValue( random.nextGaussian()*sigma+mi );
  }  
/**
 * Change the parameters of the gaussian distribution.
** @param m    Average.
** @param s    Square root of the standard deviation.
*/
  public void setParam(double m,double s)
  {
	mi = m;
	sigma = s;
  }  
/**
 * Returns the type of the extracted value.
 */
  public int type()
  {
  	return REAL_TYPE;
  }  
}