
package simula.random;

import java.util.Random;

/**
 * The class Randint allows to extract uniformely from a known set of natural consecutive
 * numbers (e.g. {4, 5, 6, 7});
*/
public class Randint extends Generator
{
/**
** Lower bound.
*/  
  private int a;

/**
** Upper bound.
*/
  private int b;


/**
 * Creates a new Randint generator using the standard engine and the current time as seed.
** @param a    Lower bound.
** @param b    Upper bound.
*/

  public Randint(int a,int b)
  {
	random = new RandomEngineJava();
	setParam(a,b);
  }  
/**
 * Creates a new Randint generator using the standard engine and the given seed.
** @param seed Initial seed for the pseudo-random engine.
** @param a    Lower bound.
** @param b    Upper bound.
*/
  public Randint(long seed,int a,int b)
  {
	random = new RandomEngineJava(seed);
	setParam(a,b);
  }  
/**
 * Creates a new Randint generator using the given engine.
** @param e    Pseudo-random bit generator.
** @param a    Lower bound.
** @param b    Upper bound.
*/
  public Randint(RandomEngine e,int a,int b)
  {
	random = e;
	setParam(a,b);
  }  
/**
 * Returns the type of the extracted value.
 */
  public double density(int x)
  {
  	double res = 0.0;
  	if ((a<=x) &&(x<=b))
  		res = 1.0/(b-a+1);
  	return res;
  }  
/**
 * Draws a value from the generator.
 * @return An instance of Value that encapsulates an integer value.
 * @see IntegerValue
 * @see Value
 */
  public Value draw()
  {
	return new IntegerValue( (int)Math.floor(random.nextDouble() * (b-a) + a) );
  }  
/**
** Change the bounds.
** @param a    Lower bound.
** @param b    Upper bound.
*/
  public void setParam(int a,int b)
  {
	this.a = a<=b ? a : b;
	this.b = a<=b ? b : a;
	this.b++;
  }  
  public int type()
  {
  	return INTEGER_TYPE;
  }  
}