
package simula.random;

import java.net.*;
import java.io.*;

/**
 * Implements a pseudo-random generator using the data collected from the
 * generator of random sequences obtained from the radioactive deplection called
** <a href="http://www.fourmilab.ch/hotbits/">HotBits</a>
** by <a href="http://www.fourmilab.ch/">John Walker</a>.
 * Suggested only for very critical application.
*/
public class RandomEngineHotBits extends RandomEngine {
	long state;
	int nuflen = 256, buflen = 0;

/**
 * Buffer of the random bits collected by HotBits.
*/
	private byte[] buffer;
	int bufptr = -1;


//  Costruttori

/**
 * Create a new HotBits generator.
 */
	public RandomEngineHotBits() {
		buffer = new byte[nuflen];
	}
/**
 * Fills the internal buffer with bits coming from HotBits.
*/  
	private void fillBuffer() throws java.io.IOException
	{
		URL u = new URL("http://www.fourmilab.ch/cgi-bin/uncgi/Hotbits?nbytes=128&fmt=bin");
		InputStream s = u.openStream();
		int l;

		buflen = 0;
		while ((l = s.read()) != -1) {
			buffer[buflen++] = (byte) l;
		}
		s.close();
		bufptr = 0;
	}
/** 
 * Returns the next byte from the generator.
** @return the next byte from the generator.
*/
	public byte nextByte()  {
		try {
			synchronized (buffer) {
				if (bufptr < 0 || bufptr >= buflen) {
					fillBuffer();
				}
				return buffer[bufptr++];
			}
		} catch (IOException e) {
			throw new RuntimeException("Non posso ottenere HotBits");
		}
	}
}