
package simula.random;

/**
** Implements "two sequences" L'Ecuyer generator
** with Bays-Durham shuffle.
** This is made possible by Java 64 bit arithmethic.
*/

public class RandomEngineLEcuyer extends RandomEngine {

/**
** Mods and operands indicated by
** L'Ecuyer for the LCG 
*/
	private static final long mul1 = 40014;
	private static final long mod1 = 2147483563;
	private static final long mul2 = 40692;
	private static final long mod2 = 2147483399;
/**
** Shuffle table dimension
*/    
	private static final int shuffleSize = 32;
/** 
** Numbers to "burn" during system warmup
*/  private static final int warmup = 19;            

	private int gen1, gen2, state;
	
/** 
** shuffle table.
*/
	private int[] shuffle;


	//  Constructors

/**
** Creates a new engine using system time as the seed
*/
	public RandomEngineLEcuyer() {
		shuffle = new int[shuffleSize];
		this.setSeed(System.currentTimeMillis());
	}
/** 
** Creates a new engine with the given seed.
** @param seed initial seed for the generator (shouldn't be NULL).
*/
	public RandomEngineLEcuyer(long seed) throws IllegalArgumentException {
		shuffle = new int[shuffleSize];
		this.setSeed(seed);
	}
/** 
** Return the next byte from this engine
** @return next byte from engine
*/
	public byte nextByte() {
		int i;

		gen1 = (int) ((gen1 * mul1) % mod1); 
		gen2 = (int) ((gen2 * mul2) % mod2); 


		i = state / (1 + (((int) mod1) - 1) / shuffleSize);

		state = (int) ((((long) shuffle[i]) + gen2) % mod1);

		shuffle[i] = gen1;

		return (byte) (state / (1 + (((int) mod1) - 1) / 256));
	}
// Seed set up 

/**
** Resets the seed for this engine
** @param seed Engine seed
*/
	public void setSeed(long seed) throws IllegalArgumentException {
		int i;

		if (seed == 0) {
			throw new IllegalArgumentException("seed must be nonzero");
		}
		gen1 = gen2 = (int) (seed & 0x7FFFFFFFFL);

		// system "Warmup" to clean off any seed influence
		  

		for (i = 0; i < warmup; i++) {
			gen1 = (int) ((gen1 * mul1) % mod1);
		}

		// shuffle table fill up

		for (i = 0; i < shuffleSize; i++) {
			gen1 = (int) ((gen1 * mul1) % mod1);
			shuffle[(shuffleSize - 1) - i] = gen1;
		}
		state = shuffle[0];
	}
}